import React, { Component } from 'react';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

// Reference: https://en.wikipedia.org/wiki/Publish-subscribe_pattern;
// Reference: https://en.wikipedia.org/wiki/Mediator_pattern;

var Hubx = function () {
  function Hubx() {
    classCallCheck(this, Hubx);

    this.observers = {};
    this.buffer = {};
    this.mediators = {};
    this.isReplaying = false;
  }

  /**
   *
   * @param {string} event Nome do evento que será escutado
   * @param {function} observer Função de callback que será chamada
   * @param {boolean} withLastValue Caso seja true durante o attach caso tenha alguma mensagem na fila
   *  a última já é passada para o observer
   */

  createClass(Hubx, [{
    key: "attach",
    value: function attach(event, observer, withLastValue) {
      var _this = this;

      if (!this.observers[event]) {
        this.observers[event] = [];
      }
      if (!this.buffer[event]) {
        this.buffer[event] = [];
      }

      if (typeof observer === "undefined") {
        return;
      }

      this.observers[event].push(observer);

      if (withLastValue === true) {
        // Caso já tenha algo na fila já dispara o callback
        if (this.buffer[event] && this.buffer[event].length > 0) {
          setTimeout(function () {
            var last = _this.buffer[event].length - 1;
            observer(_this.buffer[event][last]);
          }, 1);
        }
      }
    }
  }, {
    key: "detach",
    value: function detach(event, observer) {
      if (!this.observers[event]) {
        return;
      }
      var index = -1;
      for (var i = 0; i < this.observers[event].length; i++) {
        if (this.observers[event][i] === observer) {
          index = i;
          break;
        }
      }
      if (index >= 0) {
        this.observers[event].splice(index, 1);
      }
    }
  }, {
    key: "detachAll",
    value: function detachAll(event) {
      if (!this.observers[event]) {
        return;
      }
      this.observers[event].length = 0;
    }
  }, {
    key: "detachAllWithPrefix",
    value: function detachAllWithPrefix(prefix) {
      var _this2 = this;

      var events = Object.keys(this.observers).filter(function (s) {
        return s.startsWith(prefix);
      });
      events.forEach(function (e) {
        return _this2.observers[e].length = 0;
      });
    }
  }, {
    key: "attachMediator",
    value: function attachMediator(address, callback) {
      if (!this.mediators[address]) {
        this.mediators[address] = [];
      }
      this.mediators[address].push(callback);
    }
  }, {
    key: "request",
    value: function request(address, message) {
      if (!this.mediators[address]) {
        return new Promise(function (res, rej) {
          return rej("address not registered!");
        });
      }
      return Promise.all(this.mediators[address].map(function (x) {
        return x(message);
      }));
    }
  }, {
    key: "notify",
    value: function notify(event, message) {
      if (!this.buffer[event]) {
        this.buffer[event] = [];
      }
      this.buffer[event].pop();
      this.buffer[event].push(message);
      if (!this.observers[event]) {
        return;
      }
      var promises = [];
      this.observers[event].forEach(function (observer) {
        promises.push(new Promise(function (res, rej) {
          setTimeout(function () {
            if (typeof observer === "function") {
              try {
                observer(message);
                res();
              } catch (e) {
                rej(observer);
              }
            }
          }, 1);
        }));
      });
      return Promise.all(promises);
    }
  }, {
    key: "replay",
    value: function replay(event, rollback) {
      console.warn("untested, not production ready");
      if (this.buffer[event]) {
        var last = this.buffer.length - 1;
        for (var i = last - rollback; i <= last; i++) {
          this.notify(event, this.buffer[event][i]);
        }
      }
    }
  }]);
  return Hubx;
}();


function withHubx(WrappedComponent) {
  if (!window.Hubx) {
    window.Hubx = new Hubx();
  }
  return function (_Component) {
    inherits(_class, _Component);

    function _class(props) {
      classCallCheck(this, _class);

      var _this3 = possibleConstructorReturn(this, (_class.__proto__ || Object.getPrototypeOf(_class)).call(this, props));

      _this3.attach = _this3.attach.bind(_this3);
      _this3.notify = _this3.notify.bind(_this3);
      _this3.toDetach = {};
      return _this3;
    }

    createClass(_class, [{
      key: "attach",
      value: function attach(event, subscriber, withInitialValue) {
        if (!this.toDetach[event]) {
          this.toDetach[event] = [];
        }
        this.toDetach[event].push(subscriber);
        window.Hubx.attach(event, subscriber, withInitialValue);
      }
    }, {
      key: "detach",
      value: function detach(event, subscriber) {
        window.Hubx.detach(event, subscriber);
      }
    }, {
      key: "notify",
      value: function notify(event, message) {
        window.Hubx.notify(event, message);
      }
    }, {
      key: "request",
      value: function request(address, message) {
        return window.Hubx.request(address, message);
      }
    }, {
      key: "componentWillUnmount",
      value: function componentWillUnmount() {
        var _this4 = this;

        Object.keys(this.toDetach).map(function (event) {
          return _this4.toDetach[event].forEach(function (e) {
            window.Hubx.detach(event, e);
          });
        });
      }
    }, {
      key: "render",
      value: function render() {
        return React.createElement(WrappedComponent, _extends({}, this.props, {
          attach: this.attach,
          notify: this.notify,
          request: this.request
        }));
      }
    }]);
    return _class;
  }(Component);
}

export default Hubx;
export { withHubx };
//# sourceMappingURL=index.es.js.map
